import json
import time

import deb_pkg_tools
import os
import sys
import re
import rpm
import subprocess
from prettytable import PrettyTable
from textwrap import fill
from deb_pkg_tools.version import Version
from rpmUtils.miscutils import splitFilename

import logging

import anchore_image
from configuration import AnchoreConfiguration

module_logger = logging.getLogger(__name__)

def init_analyzer_cmdline(argv, name):
    ret = {}

    if len(argv) < 4:
        print "ERROR: invalid input"
        raise Exception

    anchore_conf = AnchoreConfiguration()
    ret['anchore_config'] = anchore_conf.data

    ret['name'] = name
    ret['imgid'] = argv[1]

    ret['dirs'] = {}
    ret['dirs']['datadir'] = argv[2]
    ret['dirs']['outputdir'] = '/'.join([argv[3], "analyzer_output", name])
    ret['dirs']['unpackdir'] = argv[4]

    for d in ret['dirs'].keys():
        if not os.path.isdir(ret['dirs'][d]):
            try:
                os.makedirs(ret['dirs'][d])
            except Exception as err:
                print "ERROR: cannot find/create input dir '"+ret['dirs'][d]+"'"
                raise err

    return(ret)

def init_gate_cmdline(argv, paramhelp):
    return(init_query_cmdline(argv, paramhelp))

def init_query_cmdline(argv, paramhelp):
    ret = {}

    if len(argv) == 2 and re.match(".*help.*", argv[1]):
        # print argv[0].split('/')[-1]
        print paramhelp
        return (False)

    if len(argv) < 5:
        print "ERROR: invalid input"
        raise Exception

    anchore_conf = AnchoreConfiguration()
    ret['anchore_config'] = anchore_conf.data

    ret['name'] = argv[0].split('/')[-1]
    ret['imgfile'] = argv[1]

    images = read_plainfile_tolist(ret['imgfile'])
    ret['imgid'] = images[0]
    if 'imgid' not in ret:
        print "ERROR: could not read imgid from input file"
        raise Exception

    ret['images'] = images

    ret['dirs'] = {}
    ret['dirs']['datadir'] = argv[2]

    ret['dirs']['imgdir'] = '/'.join([ret['dirs']['datadir'], ret['imgid'], 'image_output'])
    ret['dirs']['analyzerdir'] = '/'.join([ret['dirs']['datadir'], ret['imgid'], 'analyzer_output'])
    ret['dirs']['comparedir'] = '/'.join([ret['dirs']['datadir'], ret['imgid'], 'compare_output'])
    ret['dirs']['gatesdir'] = '/'.join([ret['dirs']['datadir'], ret['imgid'], 'gates_output'])

    ret['dirs']['outputdir'] = argv[3]

    try:
        ret['params'] = argv[4:]
    except:
        ret['params'] = list()

    for d in ret['dirs'].keys():
        thedir = ret['dirs'][d]
        if not os.path.exists(thedir):
            raise Exception(d + " directory '" + thedir + "' does not exist.")

    metafile = '/'.join([ret['dirs']['imgdir'], 'image_info', 'image.meta'])
    if not os.path.exists(metafile):
        raise Exception("image metadata not available")


    ret['meta'] = read_kvfile_todict(metafile)
    ret['imgtags'] = ret['meta']['humanname']
    ret['output'] = '/'.join([ret['dirs']['outputdir'], ret['name']])

    return (ret)


def print_result(config, result, outputmode=None):
    if not result:
        return (False)

    if not outputmode:
        if config.cliargs['json']:
            outputmode = 'json'
        elif config.cliargs['plain']:
            outputmode = 'plaintext'
        else:
            outputmode = 'table'

    try:
        width = int(subprocess.check_output(['stty', 'size']).split()[1]) - 10
    except:
        width = 70

    if outputmode == 'json':
        print json.dumps(result)
    else:
        output = list()
        if len(result.keys()) > 0:
            sortby = False

            # this is awkward - need better way to differentiate header from results
            for i in result.keys():
                json_dict = result[i]
                sortby = False
                for h in json_dict['result']['header']:
                    if re.match(r"^\*.*", h):
                        sortby = h

                header = json_dict['result']['header']
                if outputmode == 'table':
                    t = PrettyTable(header)
                break

            emptyresult = False
            for i in result.keys():
                json_dict = result[i]

                for orow in json_dict['result']['rows']:
                    if outputmode == 'table':
                        row = [fill(x, max(12, width / (len(orow))) ) for x in orow ]
                        t.add_row(row)
                    elif outputmode == 'plaintext':
                        row = [ re.sub("\s", ",", x) for x in orow ]
                        output.append(row)
                    elif outputmode == 'raw':
                        output.append(orow)
            if outputmode == 'table':
                if sortby:
                    print t.get_string(sortby=sortby, reversesort=True)
                else:
                    print t
                print ""
            elif outputmode == 'plaintext':
                print ' '.join(header)
                print ""
                for r in output:
                    print ' '.join(r)
            elif outputmode == 'raw':
                for i in range(len(output)):
                    row = output[i]
                    for j in range(len(row)):
                        print "--- " + header[j] + " ---"
                        print output[i][j]
                        print 


    return (True)

def cve_load_data(cvedatadir, image):
    cve_data = None

    distro = image.get_distro()
    distrovers = image.get_distro_vers()

    cvejsonfile = '/'.join([cvedatadir, distro+":"+distrovers, "cve.json"])
    if os.path.exists(cvejsonfile):
        FH = open(cvejsonfile, 'r')
        cve_data = json.loads(FH.read())
        FH.close()
    return (cve_data)


def cve_scanimage(cve_data, image):
    if not cve_data:
        return ({})

    all_packages = {}
    analysis_report = image.get_analysis_report().copy()

    thelist = []
    if 'package_list' in analysis_report and 'pkgs.all' in analysis_report['package_list']:
        thelist = analysis_report['package_list']['pkgs.all']
    for l in thelist:
        l = l.strip()
        (p, v) = l.split()
        if p not in all_packages:
            all_packages[p] = v

    thelist = []
    if 'package_list' in analysis_report and 'pkgs_plus_source.all' in analysis_report['package_list']:
        thelist = analysis_report['package_list']['pkgs_plus_source.all']
    for l in thelist:
        l = l.strip()
        (p, v) = l.split()
        if p not in all_packages:
            all_packages[p] = v

    results = {}
    for v in cve_data:
        outel = {}
        vuln = v['Vulnerability']
        print "cve-scan: CVE: " + vuln['Name']
        if 'FixedIn' in vuln:
            for fixes in vuln['FixedIn']:
                isvuln = False
                vpkg = fixes['Name']
                print "cve-scan: Vulnerable Package: " + vpkg
                if vpkg in all_packages:
                    ivers = all_packages[fixes['Name']]
                    vvers = re.sub(r'^[0-9]*:', '', fixes['Version'])
                    print "cve-scan: " + vpkg + "\n\tfixed vulnerability package version: " + vvers + "\n\timage package version: " + ivers
                    if image.get_distro() == "centos" or image.get_distro() == "rhel":
                        if vvers != 'None':
                            fixfile = vpkg + "-" + vvers + ".rpm"
                            imagefile = vpkg + "-" + ivers + ".rpm"
                            (n1, v1, r1, e1, a1) = splitFilename(imagefile)
                            (n2, v2, r2, e2, a2) = splitFilename(fixfile)
                            if rpm.labelCompare(('1', v1, r1), ('1', v2, r2)) < 0:
                                isvuln = True
                        else:
                            isvuln = True

                    elif image.get_distro() == "ubuntu" or image.get_distro() == "debian":
                        if vvers != 'None':                            
                            if ivers != vvers and deb_pkg_tools.version.compare_versions(ivers, '<', vvers):
                                isvuln = True
                        else:
                            isvuln = True

                    if isvuln:
                        print "cve-scan: Found vulnerable package: " + vpkg
                        severity = url = description = 'Not Available'
                        if 'Severity' in vuln:
                            severity = vuln['Severity']
                        if 'Link' in vuln:
                            url = vuln['Link']
                        if 'Description' in vuln:
                            description = vuln['Description']
                        
                        outel = {'pkgName': vpkg, 'imageVers': ivers, 'fixVers': vvers, 'severity': severity, 'url': url, 'description': description}
                if outel:
                    results[vuln['Name']] = outel
    return (results)


def cve_get_fixpkg(cve_data, cveId):
    retlist = list()
    for v in cve_data:
        vuln = v['Vulnerability']
        if cveId == vuln['Name']:
            if 'FixedIn' in vuln:
                for p in vuln['FixedIn']:
                    retlist.append(p['Name'])
    return (retlist)

def image_context_add(imagelist, allimages, docker_cli=None, dockerfile=None, anchore_datadir=None, tmproot='/tmp', anchore_db=None, must_be_analyzed=False, usertype=None, must_load_all=False):
    retlist = list()
    for i in imagelist:
        if i in allimages:
            retlist.append(i)
        else:
            try:
                newimage = anchore_image.AnchoreImage(i, anchore_datadir, docker_cli=docker_cli, allimages=allimages, dockerfile=dockerfile, tmpdirroot=tmproot, usertype=usertype, anchore_db=anchore_db)
            except Exception as err:
                if must_load_all:
                    import traceback
                    traceback.print_exc()
                    errorstr = "Could not load/initialize all input images.\n" + "\tImage: " + str(i) + "\n\tInfo: " + str(err.message)
                    raise Exception(errorstr)

            if not must_be_analyzed or newimage.is_analyzed():
                allimages[newimage.meta['imageId']] = newimage
                retlist.append(newimage.meta['imageId'])

            if must_be_analyzed and not newimage.is_analyzed():
                errorstr = "Image(s) must be analyzed before operation can be performed.\n\tImage: " + str(i)
                raise Exception(errorstr)

            #elif must_be_analyzed and not newimage.is_analyzed():
            #    raise Exception(errorstr)
            # Check that all input images are successfully loaded
            #if must_load_all and (len(retlist) != len(imagelist)):
            #    lnames = list()
            #    for i in retlist:
            #        lnames.append(allimages[i].meta['imagename'])
            #    ldiff = list(set(imagelist) - set(lnames))
            #    raise Exception(errorstr)

    return (retlist)


def update_file_list(listbuf, outfile, backup=False):
    src = listbuf
    if not os.path.exists(outfile):
        write_plainfile_fromlist(outfile, src)
        #FH = open(outfile, 'w')
        #for l in src:
        #    FH.write(l + "\n")
        #FH.close()
    else:
        dst = read_plainfile_tolist(outfile)
        #FH = open(outfile, 'r')
        #dst = list()
        #for l in FH.readlines():
        #    l = l.strip()
        #    dst.append(l)
        #FH.close()
        if src != dst:
            if backup:
                hfile = outfile + "." + str(int(time.time()))
                os.rename(outfile, hfile)

            write_plainfile_fromlist(outfile, src)
            #FH = open(outfile, 'w')
            #for l in src:
            #    FH.write(l + "\n")
            #FH.close()

    return (True)


def update_file_jsonstr(jsonbuf, outfile, backup=False):
    src = json.loads(jsonbuf)
    if not os.path.exists(outfile):
        FH = open(outfile, 'w')
        FH.write(json.dumps(src))
        FH.close()
    else:
        FH = open(outfile, 'r')
        dst = json.loads(FH.read())
        FH.close()
        if src != dst:
            if backup:
                hfile = outfile + "." + str(int(time.time()))
                os.rename(outfile, hfile)
            FH = open(outfile, 'w')
            FH.write(json.dumps(src))
            FH.close()

    return (True)


def update_file_str(buf, outfile, backup=False):
    src = buf
    if not os.path.exists(outfile):
        write_plainfile_fromstr(outfile, src)
        #FH = open(outfile, 'w')
        #FH.write(src)
        #FH.close()
    else:
        dst = read_plainfile_tostr(outfile)
        #FH = open(outfile, 'r')
        #dst = FH.read()
        #FH.close()
        if src != dst:
            if backup:
                hfile = outfile + "." + str(int(time.time()))
                os.rename(outfile, hfile)
            write_plainfile_fromstr(outfile, src)
            #FH = open(outfile, 'w')
            #FH.write(src)
            #FH.close()

    return (True)


def write_plainfile_fromstr(file, instr):
    FH=open(file, 'w')
    thestr = instr.encode('utf8')
    FH.write(thestr)
    FH.close()

def read_plainfile_tostr(file):
    if not os.path.isfile(file):
        return ("")
    FH = open(file, 'r')
    ret = FH.read().decode('utf8')
    FH.close()
    return (ret)


def read_kvfile_tolist(file):
    if not os.path.isfile(file):
        return([])

    ret = list()
    FH=open(file, 'r')
    for l in FH.readlines():
        l = l.strip().decode('utf8')
        row = l.split()
        ret.append(row)
    FH.close()

    return (ret)

def read_plainfile_tolist(file):
    if not os.path.isfile(file):
        return([])

    ret = list()
    FH=open(file, 'r')
    for l in FH.readlines():
        l = l.strip().decode('utf8')
        ret.append(l)
    FH.close()

    return (ret)

def read_kvfile_todict(file):
    if not os.path.isfile(file):
        return ({})

    ret = {}
    FH = open(file, 'r')
    for l in FH.readlines():
        l = l.strip().decode('utf8')
        (k, v) = re.match('(\S*)\s*(.*)', l).group(1, 2)
        ret[k] = v
    FH.close()

    return (ret)

def write_plainfile_fromlist(file, list):
    OFH = open(file, 'w')
    for l in list:
        thestr = l + "\n"
        thestr = thestr.encode('utf8')
        OFH.write(thestr)
    OFH.close()

def write_kvfile_fromlist(file, list):
    OFH = open(file, 'w')
    for l in list:
        thestr = ' '.join(l) + "\n"
        thestr = thestr.encode('utf8')
        OFH.write(thestr)
    OFH.close()

def write_kvfile_fromdict(file, indict):
    dict = indict.copy()
    OFH = open(file, 'w')
    for k in dict.keys():
        if not dict[k]:
            dict[k] = "none"
        thestr = ' '.join([k, dict[k], '\n'])
        thestr = thestr.encode('utf8')
        OFH.write(thestr)
    OFH.close()

def touch_file(file):
    return(open(file, 'a').close())



#!/usr/bin/env python

import sys
import os
import json
import re
import rpm
from rpmUtils.miscutils import splitFilename
import deb_pkg_tools
from deb_pkg_tools.version import Version
from anchore import anchore_image
import anchore.anchore_utils

try:
    config = anchore.anchore_utils.init_gate_cmdline(sys.argv, "CVE Checking Gate")
except Exception as err:
    print str(err)
    sys.exit(1)

if not config:
    print "ERROR: could not set up environment for gate"
    sys.exit(1)

imgid = config['imgid']
imgdir = config['dirs']['imgdir']
analyzerdir = config['dirs']['analyzerdir']
comparedir = config['dirs']['comparedir']
outputdir = config['dirs']['outputdir']

try:
    params = config['params']
except:
    params = None

cvedirroot = '/'.join([config['anchore_config']['image_data_store'], "../cve-data"])

try:
    image = anchore_image.AnchoreImage(imgid, config['anchore_config']['image_data_store'], {})
    cve_data = anchore.anchore_utils.cve_load_data(cvedirroot, image)
    report = anchore.anchore_utils.cve_scanimage(cve_data, image)
except Exception as err:
    print "ERROR: could not scan image for CVEs: " + str(err)
    exit(1)

output = '/'.join([outputdir, 'ANCHORESEC'])
OFH=open(output, 'w')
for k in report.keys():
    vuln = report[k]
    cve = k
    pkg = vuln['pkgName']
    sev = vuln['severity']
    url = vuln['url']
    if sev == 'Low':
        t = "VULNLOW"
    elif sev == 'Medium':
        t = "VULNMEDIUM"
    elif sev == "High":
        t = "VULNHIGH"
    elif sev == "Critical":
        t = "VULNCRITICAL"
    else:
        t = "VULNUNKNOWN"

    OFH.write(t + " " + sev + " Vulnerability found in package - " + pkg + " (" + cve + " - " + url + ")\n")

OFH.close()
sys.exit(0)



import os
import subprocess
import re
import logging


class ScriptSetExecutor:
    """
    A script module is a module for executing a set of user-provided scripts found in a specific directory in
    lexicographically sorted order

    """
    _logger = logging.getLogger(__name__)

    def __init__(self, path, file_prefix='', ascending=True):
        assert path is not None

        self.inputdir = path
        self.prefix = file_prefix
        self.sort_ascending = ascending

    def check(self, init_if_missing=False):
        """
        Check the path and construct if not found and init_if_missing=True
        :param init_if_missing: create the path if not found
        :return: true if exists false if not
        """

        if os.path.exists(self.inputdir):
            return True
        elif init_if_missing:
            os.makedirs(self.inputdir)
            return True

    def get_scripts(self, fullpath=False):
        """
        Get the scripts at the path in sorted order as set in the module properties
        :return: a sorted list of scripts
        """
        scripts = filter(lambda x: x.startswith(self.prefix), os.listdir(self.inputdir))
        scripts.sort(reverse=(not self.sort_ascending))
        if fullpath:
            return [os.path.join(self.inputdir, x) for x in scripts]
        else:
            return scripts

    def execute(self, capture_output=False, fail_fast=False, cmdline=None, **kwargs):
        """
        Pass in the kwargs as --<name> <value> pairs.
        :param capture_output: if True then return output of script in return dict. If False, only return code
        :param fail_fast:if True then quit execution after first non-zero return code, else execute all
        :returns list of (script, returncode, stdout) tuples in order of execution
        """

        scripts = self.get_scripts()
        scripts = [os.path.join(self.inputdir, x) for x in scripts]
        output = []

        for script in scripts:
            if not os.access(script, os.R_OK ^ os.X_OK) or re.match(".*~$", script):
                # Skip any that are not executable (e.g. README or txt files)
                continue

            self._logger.debug('Executing script: %s' % script)

            if cmdline:
                cmd = [script] + cmdline.split()
            else:
                cmd = [script]

            if capture_output:
                try:
                    output.append((' '.join(cmd), 0, subprocess.check_output(cmd, stderr=subprocess.STDOUT, **kwargs)))
                except subprocess.CalledProcessError as e:
                    output.append((' '.join(cmd), e.returncode, e.output))
            else:
                output.append((' '.join(cmd), subprocess.call(cmd, **kwargs), None))

            self._logger.debug('%s return code %d, output: %s' % output[-1])

            if fail_fast and output[-1][1]:
                break

        return output





import sys
import os
import re
import shutil
from textwrap import fill
import click

from anchore.cli.common import anchore_print, anchore_print_err
from anchore import visualizer, navigator, anchore_utils
from anchore.util import contexts

config = {}
imagelist = []

@click.group(short_help='Useful tools and operations on images and containers')
@click.option('--image', help='Process specified image ID', required=True, metavar='<imageid>')
@click.pass_obj
def toolbox(anchore_config, image):
    """
    A collection of tools for operating on images and containers and building anchore modules.

    Subcommands operate on the specified image passed in as --image <imgid>

    """
    global config, imagelist, nav
    config = anchore_config
    ecode = 0

    imagelist = [image]

    try:
        nav = navigator.Navigator(anchore_config=config, imagelist=imagelist, allimages=contexts['anchore_allimages'])
    except Exception as err:
        anchore_print_err('operation failed')
        nav = None
        ecode = 1


@toolbox.command(name='unpack')
@click.option('--destdir', help='Destination directory for unpacked container image', metavar='<path>')
def unpack(destdir):
    """Unpack and Squash image to local filesystem"""

    if not nav:
        sys.exit(1)

    ecode = 0
    try:
        anchore_print("Unpacking images: " + ' '.join(nav.get_images()))
        result = nav.unpack(destdir=destdir)
        for imageId in result:
            anchore_print("Unpacked image: " + imageId)
            anchore_print("Unpack directory: "+ result[imageId])
    except:
        anchore_print_err("operation failed")
        ecode = 1

    contexts['anchore_allimages'].clear()
    
    sys.exit(ecode)

@toolbox.command(name='setup-module-dev', short_help='Setup a module development environment')
@click.option('--destdir', help='Destination directory for module development environment')
def setup_module_dev(destdir):
    """
    Sets up a development environment suitable for working on anchore modules (queries, etc) in the specified directory.
    Creates a copied environment in the destination containing the module scripts, unpacked image(s) and helper scripts
    such that a module script that works in the environment can be copied into the correct installation environment and
    run with anchore explore <modulename> invocation and should work.

    """

    if not nav:
        sys.exit(1)

    ecode = 0
    try:
        anchore_print("Anchore Module Development Environment\n")
        helpstr = "This tool has set up and environment that represents what anchore will normally set up before running and analyzer, gate and/or query module.  Each section below includes some information along with a string that you can use to help develop your own anchore modules.\n"
        anchore_print(fill(helpstr, 80))
        anchore_print("")

        anchore_print("Setting up environment...")
        anchore_print("")

        result = nav.unpack(destdir=destdir)
        for imageId in result:
            unpackdir = result[imageId]

            # copy anchore imageDB dir into unpacked environment
            imgdir = '/'.join([config.data['image_data_store'], imageId])
            tmpdatastore = '/'.join([unpackdir, 'data'])
            dstimgdir = '/'.join([tmpdatastore, imageId])

            if not os.path.exists(imgdir):
                anchore_print_err("Image must exist and have been analyzed before being used for module development.")
                break
            if not os.path.exists(tmpdatastore):
                os.makedirs(tmpdatastore)
            shutil.copytree(imgdir, dstimgdir, symlinks=True)

            # copy examples into the unpacked environment            
            examples = {}
            basedir = '/'.join([unpackdir, "anchore-modules"])
            if not os.path.exists(basedir):
                os.makedirs(basedir)

                # copy the shell-utils
                os.makedirs('/'.join([basedir, 'shell-utils']))
                for s in os.listdir('/'.join([config.data['scripts_dir'], 'shell-utils'])):
                    shutil.copy('/'.join([config.data['scripts_dir'], 'shell-utils', s]), '/'.join([basedir, 'shell-utils', s]))
                            
            # copy any examples that exist in the anchore egg into the unpack dir
            for d in os.listdir(config.data['scripts_dir']):
                scriptdir = '/'.join([basedir, d])

                if os.path.exists(config.data['scripts_dir'] + "/examples/" + d):
                    if not os.path.exists(scriptdir):
                        os.makedirs(scriptdir)
                    for s in os.listdir(config.data['scripts_dir'] + "/examples/" + d):
                        thefile = '/'.join([config.data['scripts_dir'], "examples", d, s])
                        thefiledst = '/'.join([scriptdir, s])
                        if re.match(".*(\.sh)$", thefile):
                            examples[d] = thefiledst
                            shutil.copy(thefile, thefiledst)

            # all set, show how to use them
            anchore_print("\tImage: " + imageId[0:12])
            anchore_print("\tUnpack Directory: " +result[imageId])
            anchore_print("")
            #analyzer_string = ' '.join([examples['analyzers'], imageId, config.data['image_data_store'], config.data['image_data_store'] + "/" + imageId, result[imageId]])
            analyzer_string = ' '.join([examples['analyzers'], imageId, tmpdatastore, dstimgdir, result[imageId]])
            anchore_print("\tAnalyzer Command:\n\n\t" +analyzer_string)
            anchore_print("")

            anchore_utils.write_plainfile_fromstr(result[imageId] + "/queryimages", imageId+"\n")

#            gate_string = ' '.join([examples['gates'], result[imageId] + "/queryimages", config.data['image_data_store'], config.data['image_data_store'] + "/"+imageId+"/gates_output", "all"])
#            anchore_print("***Gate***\n")
#            anchore_print("\tCommand:\n\n\t" + gate_string)
#            anchore_print("")

            queryoutput = '/'.join([result[imageId], "querytmp/"])
            if not os.path.exists(queryoutput):
                os.makedirs(queryoutput)

            #query_string = ' '.join([examples['queries'], result[imageId] + "/queryimages", config.data['image_data_store'], queryoutput, "passwd"])
            query_string = ' '.join([examples['queries'], result[imageId] + "/queryimages", tmpdatastore, queryoutput, "passwd"])
            anchore_print("Query Command:\n\n\t" + query_string)
            anchore_print("")
 
            anchore_print("Next Steps: ")
            anchore_print("\tFirst: run the above analyzer command and note the RESULT output")
            anchore_print("\tSecond: run the above query command and note the RESULT output, checking that the query was able to use the analyzer data to perform its search")
            anchore_print("\tThird: modify the analyzer/query modules as you wish, including renaming them and continue running/inspecting output until you are satisfied")
            anchore_print("\tFinally: when you're happy with the analyzer/query, copy them to next to existing anchore analyzer/query modules and anchore will start calling them as part of container analysis/query:\n")
            anchore_print("\tcp " + examples['analyzers'] + " " + config.data['scripts_dir'] + "/analyzers/99_analyzer-example.sh")
            anchore_print("\tcp " + examples['queries'] + " " + config.data['scripts_dir'] + "/queries/")
            anchore_print("\tanchore analyze --force --image " + imageId + " --imagetype none")
            anchore_print("\tanchore explore --image " + imageId + " query query-example")
            anchore_print("\tanchore explore --image " + imageId + " query query-example passwd")
            anchore_print("\tanchore explore --image " + imageId + " query query-example pdoesntexist")
            
    except:
        anchore_print_err("operation failed")
        ecode = 1

    contexts['anchore_allimages'].clear()
    
    sys.exit(ecode)


@toolbox.command(name='show-dockerfile')
def generate_dockerfile():
    """Generate (or display actual) image Dockerfile"""

    if not nav:
        sys.exit(1)

    ecode = 0
    try:
        result = nav.get_dockerfile_contents()
        if result:
            anchore_utils.print_result(config, result, outputmode='raw')

    except:
        anchore_print_err("operation failed")
        ecode = 1

    contexts['anchore_allimages'].clear()
    sys.exit(ecode)



@toolbox.command(name='show-layers')
def show_layers():
    """Show image layer IDs"""

    if not nav:
        sys.exit(1)

    ecode = 0
    try:
        result = nav.get_layers()
        if result:
            anchore_utils.print_result(config, result)

    except:
        anchore_print_err("operation failed")
        #import traceback
        #traceback.print_exc()
        ecode = 1

    contexts['anchore_allimages'].clear()
    sys.exit(ecode)


@toolbox.command(name='show-familytree')
def show_familytree():
    """Show image family tree image IDs"""
    if not nav:
        sys.exit(1)

    ecode = 0
    try:
        result = nav.get_familytree()
        if result:
            anchore_utils.print_result(config, result)

    except:
        anchore_print_err("operation failed")
        #import traceback
        #traceback.print_exc()
        ecode = 1

    contexts['anchore_allimages'].clear()    
    sys.exit(ecode)


@toolbox.command(name='show-taghistory')
def show_taghistory():
    """Show history of all known repo/tags for image"""


    if not nav:
        sys.exit(1)

    ecode = 0
    try:
        result = nav.get_taghistory()
        if result:
            anchore_utils.print_result(config, result)

    except:
        anchore_print_err("operation failed")
        ecode = 1

    contexts['anchore_allimages'].clear()
    sys.exit(ecode)




#!/usr/bin/env python

import sys
import os
import re
import anchore.anchore_utils

# main routine

try:
    config = anchore.anchore_utils.init_query_cmdline(sys.argv, "params: <package> <package> ...\nhelp: search input image(s) for specified <package> installations")
except Exception as err:
    print str(err)
    sys.exit(1)

if not config:
    sys.exit(0)

if len(config['params']) <= 0:
    print "Query requires input: <packageA> <packageB> ..."

outlist = list()
outlist.append(["ImageID", "Repo/Tag", "QueryParam", "Package", "Version"])

try:
    pkgfile = '/'.join([config['dirs']['analyzerdir'], 'package_list', 'pkgs.all'])
    FH=open(pkgfile, 'r')

    pkgs = anchore.anchore_utils.read_kvfile_todict(pkgfile)

    for p in pkgs.keys():
        pkgs[p + "-" + pkgs[p]] = pkgs[p]

    for pkg in config['params']:
        
        if pkg in pkgs:
            version = pkgs[pkg]
            outlist.append([config['meta']['shortId'], config['meta']['humanname'], pkg, pkg, version])
        else:
            #outlist.append([pkg, "NOMATCH", "NOMATCH"])
            pass

except:
    #outlist.append(["NOMATCH", "NOMATCH", "NOMATCH"])
    pass

if len(outlist) < 1:
    #outlist.append(["NOMATCH", "NOMATCH", "NOMATCH"])
    pass

anchore.anchore_utils.write_kvfile_fromlist(config['output'], outlist)

sys.exit(0)



import json
import random
import shutil
import subprocess
from textwrap import fill

import docker
import sys
import os
import re
from prettytable import PrettyTable

import logging

import anchore_image_db
import anchore_utils

DEVNULL = open(os.devnull, 'wb')

class AnchoreImage(object):
    """
    Represents a single image in the Anchore DB. On construction of the object the db is read and data loaded.
    """
    _logger = logging.getLogger(__name__)

    """ Constructors and Destructors"""

    def __del__(self):
        if self.initialized:
            self.save_image()

        if self.tmpdir and self.docleanup and os.path.exists(self.tmpdir):
            shutil.rmtree(self.tmpdir)

    def __init__(self, imagename, anchore_image_datadir, allimages, tmpdirroot="/tmp", dockerfile=None, docker_cli=None, anchore_db=None, usertype=None):
        # all members
        self.allimages = allimages
        self.initialized = False
        self.docleanup = True
        self.tmpdirroot = tmpdirroot
        self.tmpdir = '/'.join([self.tmpdirroot, str(random.randint(0, 9999999)) + ".anchoretmp"])

        self.dockerfile = dockerfile
        self.dockerfile_contents = None
        self.dockerfile_mode = None
        self.docker_cli = None
        self.docker_data = {}
        self.docker_data_json = ""

        self.meta = {'imagename': imagename,
                     'shortname': None,
                     'humanname': None,
                     'imageId': None,
                     'shortId': None,
                     'parentId': None,
                     'shortparentId': None,
                     'usertype': usertype}

        self.anchore_image_datadir = None
        self.anchore_imagedir = None

        self.anchore_data = {}
        self.anchore_data_json = ""
        self.anchore_allfiles = {}
        self.anchore_allpkgs = {}
        self.anchore_familytree = None
        self.anchore_layers = None
        self.anchore_current_tags = []
        self.anchore_all_tags = []
        self.anchore_tag_history = []

        self.anchore_analyzer_meta_json = None
        self.anchore_analyzer_meta = None

        self.anchore_analysis_report = None
        self.anchore_compare_report = None
        self.anchore_gates_report = None
        self.anchore_gates_eval_report = None
        self.anchore_image_report = None

        self.anchore_db = None

        # do some setup
        patt = re.compile('[0-9a-fA-F]+')
        if (len(self.meta['imagename']) == 64 and patt.match(self.meta['imagename'])):
            # imagename is a docker long uuid
            self.meta['shortname'] = self.meta['imagename'][0:12]
        else:
            # image name is a non-uuid or a short uuid
            self.meta['shortname'] = self.meta['imagename']

        if docker_cli:
            self.docker_cli = docker_cli
        else:
            self.docker_cli = docker.Client(base_url='unix://var/run/docker.sock')

        self.anchore_image_datadir = anchore_image_datadir
        if not os.path.exists(self.anchore_image_datadir):
            os.makedirs(self.anchore_image_datadir)

        if anchore_db:
            self.anchore_db = anchore_db
        else: 
            self.anchore_db = anchore_image_db.AnchoreImageDB(imagerootdir=self.anchore_image_datadir)

        # set up metadata about the image from anchore and docker
        if not self.load_image():
            raise Exception("could not load image from Docker or Anchore")

        # set up image directory structure
        try:
            self.outputdirs = {'image': 'image_output', 'analyzer': 'analyzer_output', 'compare': 'compare_output', 'gate': 'gates_output'}
            for d in self.outputdirs.keys():
                thedir = '/'.join([self.anchore_imagedir, self.outputdirs[d]])
                if not os.path.exists(thedir):
                    os.makedirs(thedir)
        except Exception as err:
            raise err

        # set up any additional internal members
        self.initialized = True

        self.discover_layers()
        self.discover_familytree()

        newlist = list(self.anchore_familytree)
        while self.meta['imageId'] in newlist: newlist.remove(self.meta['imageId'])
        anchore_utils.image_context_add(newlist, self.allimages, docker_cli=self.docker_cli, anchore_datadir=self.anchore_image_datadir, tmproot=self.tmpdirroot, anchore_db=self.anchore_db)

        # Dockerfile handling
        if self.dockerfile:
            shutil.copy(self.dockerfile, self.anchore_imagedir + "/Dockerfile")

        if os.path.exists(self.anchore_imagedir + "/Dockerfile"):
            self.dockerfile_contents = anchore_utils.read_plainfile_tostr(self.anchore_imagedir + "/Dockerfile")
            self.dockerfile_mode = 'Actual'
            self.meta['usertype'] = 'user'
        elif os.path.exists(self.anchore_imagedir + "/Dockerfile.guessed"):
            self.dockerfile_contents = anchore_utils.read_plainfile_tostr(self.anchore_imagedir + "/Dockerfile.guessed")
            self.dockerfile_mode = 'Guessed'
        else:
            self.dockerfile_contents = self.discover_dockerfile_contents()
            self.dockerfile_mode = 'Guessed'

    """ Image loading, discovering and saving """

    def load_image(self):
        if os.path.exists('/'.join([self.anchore_image_datadir,self.meta['imagename']])):
            self.meta['imageId'] = self.meta['imagename']

        try:
            self.load_image_from_docker()
        except:
            pass

        self.sync_image_meta()

        if not self.meta['imageId']:
            return (False)

        self.anchore_imagedir = '/'.join([self.anchore_image_datadir, self.meta['imageId']])
        if not os.path.exists(self.anchore_imagedir):
            os.makedirs(self.anchore_imagedir)

        self.load_image_from_anchore()
        self.sync_image_meta()

        return (True)

    def load_image_from_anchore(self):
        #anchore_data = anchore_image_db.AnchoreImageDB(imagerootdir=self.anchore_image_datadir).load_image(self.meta['imageId'])
        anchore_data = self.anchore_db.load_image(self.meta['imageId'])

        self.anchore_data = anchore_data.pop('meta', {})
        self.anchore_data_json = json.dumps(self.anchore_data)

        val = anchore_data.pop('all_tags', [])
        if len(val) > 0:
            for v in val:
                if v not in self.anchore_all_tags:
                    self.anchore_all_tags.append(v)

        self.anchore_allfiles = anchore_data.pop('allfiles', {})

        self.anchore_allpkgs = anchore_data.pop('allpkgs', {})

        val = anchore_data.pop('familytree', [])
        if len(val) > 0:
            self.anchore_familytree = val

        val = anchore_data.pop('layers', [])
        if len(val) > 0:
            self.anchore_layers = val

        val = anchore_data.pop('tag_history', [])
        if len(val) > 0:
            self.anchore_tag_history = val

        self.anchore_analyzer_meta = anchore_data.pop('analyzer_meta', {})

        self.anchore_other = {}
        if (len(anchore_data.keys()) > 0):
            self.anchore_other = anchore_data.copy()

        return (True)

    def load_image_from_docker(self):
        imagename = self.meta['imagename']
        shortname = self.meta['shortname']

        self.docker_data = self.docker_cli.inspect_image(shortname)
        self.docker_data_json = json.dumps(self.docker_data)

        for t in self.docker_data['RepoTags']:
            if t not in self.anchore_current_tags:
                self.anchore_current_tags.append(t)
            if t not in self.anchore_all_tags:
                self.anchore_all_tags.append(t)

        return (True)

    def sync_image_meta(self):
        for k in self.anchore_data.keys():
            if not k in self.meta or not self.meta[k]:
                self.meta[k] = self.anchore_data[k]

        if self.docker_data:
            self.meta['imageId'] = self.docker_data['Id'].replace("sha256:", "", 1)
            self.meta['shortId'] = self.meta['imageId'][0:12]
            self.meta['parentId'] = self.docker_data['Parent'].replace("sha256:", "", 1)
            self.meta['shortparentId'] = self.meta['parentId'][0:12]

        self.meta['humanname'] = self.get_human_name()
        return (True)

    def save_image(self):
        # Dockerfile handling
        if self.dockerfile_contents:
            if self.dockerfile_mode == 'Guessed':
                anchore_utils.update_file_str(self.dockerfile_contents, self.anchore_imagedir + "/Dockerfile.guessed", backup=False)
            elif self.dockerfile_mode == 'Actual':
                anchore_utils.update_file_str(self.dockerfile_contents, self.anchore_imagedir + "/Dockerfile", backup=False)
                if os.path.exists(self.anchore_imagedir + "/Dockerfile.guessed"):
                    os.remove(self.anchore_imagedir + "/Dockerfile.guessed")

        # Image output dir populate
        imageoutputdir = self.anchore_imagedir + "/image_output/image_info"
        if not os.path.exists(imageoutputdir):
            os.makedirs(imageoutputdir)

        anchore_utils.write_kvfile_fromdict(imageoutputdir + "/image.meta", self.meta)

        level = 0
        tagdict = {}
        for t in self.anchore_current_tags:
            tagdict[t] = str(level)
            level = level + 1
        anchore_utils.write_kvfile_fromdict(imageoutputdir + "/image_current.tags", tagdict)


        level = 0
        tagdict = {}
        for t in self.anchore_all_tags:
            tagdict[t] = str(level)
            level = level + 1
        anchore_utils.write_kvfile_fromdict(imageoutputdir + "/image_all.tags", tagdict)

        dfile = self.get_dockerfile()
        if dfile:
            shutil.copy(dfile, imageoutputdir + "/Dockerfile")

        if not os.path.exists(self.anchore_imagedir + "/image_output/image_familytree/"):
            os.makedirs(self.anchore_imagedir + "/image_output/image_familytree/")

        level = 0
        ldict = {}
        for fid in self.get_layers():
            ldict[fid] = str(level)
            level = level + 1
        anchore_utils.write_kvfile_fromdict(self.anchore_imagedir + "/image_output/image_familytree/layers", ldict)

        level = 0
        ldict = {}
        for fid in self.get_familytree():
            ldict[fid] = str(level)
            src = '/'.join([self.anchore_image_datadir, fid])
            dst = '/'.join([self.anchore_imagedir, "/image_output/image_familytree/", fid])
            try:
                os.remove(dst)
            except:
                pass
            os.symlink(src, dst)

            level = level + 1
            if self.get_earliest_base() == fid:
                src = '/'.join([self.anchore_image_datadir, fid])
                dst = '/'.join([self.anchore_imagedir, "/image_output/image_familytree/base"])
                try:
                    os.remove(dst)
                except:
                    pass
                os.symlink(src, dst)
        anchore_utils.write_kvfile_fromdict(self.anchore_imagedir + "/image_output/image_familytree/familytree", ldict)

        # generate and save image report
        report = self.generate_image_report()
        self.anchore_db.save_image_report(self.meta['imageId'], report)

    def discover_dockerfile_contents(self):
        dbuf = ""
        try:
            history = self.docker_cli.history(self.meta['imageId'])
        except:
            return (False)
        lbase = None
        cmds = list()
        for h in history:
            lid = cmd = None
            if 'Id' in h:
                lid = h['Id'].replace("sha256:", "", 1)
            if 'CreatedBy' in h:
                cmd = h['CreatedBy']

            if lid and cmd:
                if lid in self.allimages:
                    limage = self.allimages[lid]
                    if lid != self.meta['imageId'] and (limage.is_anchore_base() or limage.was_anchore_base()):
                        if not lbase:
                            lbase = limage.meta['imageId']
                if not lbase:
                    cmds.append(cmd)

        cmds.reverse()
        if lbase:
            limage = self.allimages[lbase]
            thetag = None
            patt = re.compile(".*latest.*")
            for t in limage.get_alltags_ever():
                if patt.match(t):
                    thetag = t
                    break
                thetag = t
            if thetag:
                dbuf = dbuf + "FROM " + thetag + "\n"
            else:
                dbuf = dbuf + "FROM <UNKNOWN>\n"
        else:
            dbuf = dbuf + "FROM scratch\n"

        if len(cmds) > 0:
            patt = re.compile(".*#\(nop\).*")
            for c in cmds:
                if not patt.match(c):
                    dbuf = dbuf + "RUN " + c + "\n"
                else:
                    c = re.sub(r"^/bin/sh -c #\(nop\) ", "", c)
                    dbuf = dbuf + c + "\n"
        return (dbuf)

    def discover_familytree(self):
        familytree = list()

        if self.anchore_familytree and len(self.anchore_familytree) > 0:
            return (True)

        nextimage = self
        done = 0
        while not done:
            image_id = nextimage.meta['imageId']
            parent_id = nextimage.meta['parentId']
            if image_id: familytree.append(image_id)
            if parent_id:
                if parent_id not in self.allimages:
                    self.allimages[parent_id] = AnchoreImage(parent_id,
                                                             anchore_image_datadir=self.anchore_image_datadir,
                                                             allimages=self.allimages, tmpdirroot=self.tmpdirroot,
                                                             docker_cli=self.docker_cli)
                nextimage = self.allimages[parent_id]
            else:
                done = 1

        self.anchore_familytree = list(familytree)
        self.anchore_familytree.reverse()

        return (True)

    def discover_layers(self):
        imagename = self.meta['imageId']
        imagedir = self.tmpdir
        layers = list()

        if self.anchore_layers and len(self.anchore_layers) > 0:
            return (True)

        imagedir = self.unpack()

        l = imagename

        skiptraverse = False
        if os.path.exists(imagedir + "/repositories"):
            inputf = imagedir + "/repositories"
            FH = open(inputf, 'r')
            json_dict = json.loads(FH.read())
            FH.close()
            l = json_dict[imagename]["latest"]
        elif (os.path.exists(imagedir + "/manifest.json")):
            inputf = imagedir + "/manifest.json"
            FH = open(inputf, 'r')
            json_dict = json.loads(FH.read())
            FH.close()
            layerfiles = json_dict[0]["Layers"]
            for layer in layerfiles:
                (l, tfile) = layer.split('/')
                layers.append(l)
                skiptraverse = True

        if not skiptraverse:
            done = 0
            while not done:
                layers.append(l)
                inputf = imagedir + "/" + l + "/json"
                FH = open(inputf, 'r')
                json_dict = json.loads(FH.read())
                if "parent" in json_dict:
                    l = json_dict["parent"]
                else:
                    done = 1
                    FH.close()

        self.anchore_layers = list(layers)

        if self.tmpdir and os.path.exists(self.tmpdir):
            shutil.rmtree(self.tmpdir)

        return (True)

    """ is_ and was_ checkers """

    # if image is or was ever and Anchore base image
    def is_base(self):
        if self.meta['usertype'] == 'base' or self.meta['usertype'] == 'anchorebase':
            return (True)
        return (False)

    def is_anchore_base(self):
        # returns true only if image is the current anchore base
        if os.path.exists(self.anchore_image_datadir + "/analysis_mapping.json"):
            FH = open(self.anchore_image_datadir + "/analysis_mapping.json", 'r')
            latest_anchore_images = json.loads(FH.read())
            FH.close()
        else:
            latest_anchore_images = self.docker_cli.images(all=True, filters={'dangling': False})

        if self.meta['usertype'] == 'anchorebase':
            for i in latest_anchore_images:
                patt = re.compile('.*' + self.meta['imageId'] + '.*')
                if patt.match(i['Id']):
                    return (True)
            self.meta['usertype'] == 'oldanchorebase'
        return (False)

    def was_anchore_base(self):
        # returns True if image was an anchore base image, but is no longer
        if self.meta['usertype'] == 'oldanchorebase':
            return (True)

        return (False)

    # if image ever was analyzed with a given dockerfile
    def is_user(self):
        if self.meta['usertype'] == 'user':
            return (True)
        return (False)

    def is_intermediate(self):
        if not self.meta['usertype'] or self.meta['usertype'] == 'none':
            return (True)
        return (False)

    def is_analyzed(self):
        return os.path.exists(self.anchore_imagedir + "/analyzers.done")

    """ get_ routines """

    def get_analysis_report(self):
        if not self.anchore_analysis_report:
            self.anchore_analysis_report = self.anchore_db.load_analysis_report(self.meta['imageId'])

        return self.anchore_analysis_report

    def get_compare_report(self):
        if not self.anchore_compare_report:
            self.anchore_compare_report = self.anchore_db.load_compare_report(self.meta['imageId'])

        return self.anchore_compare_report

    def get_image_report(self):
        if not self.anchore_image_report:
            self.anchore_image_report = self.anchore_db.load_image_report(self.meta['imageId'])

        return self.anchore_image_report

    def get_gates_report(self):
        if not self.anchore_gates_report:
            self.anchore_gates_report = self.anchore_db.load_gates_report(self.meta['imageId'])

        return self.anchore_gates_report

    def get_gates_eval_report(self):
        if not self.anchore_gates_eval_report:
            self.anchore_gates_eval_report = self.anchore_db.load_gates_eval_report(self.meta['imageId'])

        return self.anchore_gates_eval_report

    def get_distro(self):
        if not 'DISTRO' in self.anchore_analyzer_meta:
            return ("UNKNOWN")

        return (self.anchore_analyzer_meta['DISTRO'])

    def get_distro_vers(self):
        if not 'DISTROVERS' in self.anchore_analyzer_meta:
            return ("")

        return (self.anchore_analyzer_meta['DISTROVERS'])

    def get_latest_userimage(self):
        revtree = list(self.get_familytree())
        revtree.reverse()
        for i in revtree[1:]:
            image = self.allimages[i]
            if image.is_user():
                return (i)
        return (None)

    def get_earliest_anchore_base(self):
        for fid in self.get_familytree():
            if fid in self.allimages.keys():
                fimage = self.allimages[fid]
                if fimage.is_base():
                    return (fid)
        return None

    def get_earliest_base(self):
        # for i in self.get_familytree():
        #    image = self.allimages[i]
        #    if image.is_base():
        #        return(image['imageId'])
        # return(None)
        return (self.anchore_familytree[0])

    def get_allfiles(self):
        if not self.anchore_allfiles:
            self.load_image_from_anchore()

        return (self.anchore_allfiles)

    def get_allpkgs(self):
        if not self.anchore_allpkgs:
            self.load_image_from_anchore()

        return (self.anchore_allpkgs)

    def get_human_string(self):
        return (self.meta['shortId'] + " (" + self.get_human_name() + ")")

    def get_human_name(self):
        if self.meta['humanname']:
            return (self.meta['humanname'])

        alltags = self.get_alltags_ever()
        if len(alltags) > 0:
            patt = re.compile(".*latest.*")
            for t in alltags:
                othertag = t
                if patt.match(t):
                    self.meta['humanname'] = t
                    return (self.meta['humanname'])

            self.meta['humanname'] = othertag
            return (self.meta['humanname'])
        return (self.meta['shortId'])

    # gets list of all tags that anchore has ever seen
    def get_alltags_ever(self):
        return (self.anchore_all_tags)

    # gets all known tags from past (excluding current tags) anchore analysis data
    def get_alltags_past(self):
        return (list(set(self.anchore_all_tags) - set(self.anchore_current_tags)))

    # gets all current tags
    def get_alltags_current(self):
        return (self.anchore_current_tags)

    def get_tag_history(self):
        return (self.anchore_tag_history)

    def get_imagedir(self):
        return (self.anchore_imagedir)

    def get_layers(self):
        return (self.anchore_layers)

    def get_usertype(self):
        return (self.meta['usertype'])

    def get_familytree(self):
        return (self.anchore_familytree)

    def get_dockerfile(self):
        if os.path.exists(self.anchore_imagedir + "/Dockerfile"):
            return (self.anchore_imagedir + "/Dockerfile")
        elif os.path.exists(self.anchore_imagedir + "/Dockerfile.guessed"):
            return (self.anchore_imagedir + "/Dockerfile.guessed")
        return (False)

    """ Utilities and report generators """

    def squash(self, imagedir=None):
        if not imagedir:
            imagedir = self.tmpdir

        rootfsdir = imagedir + "/rootfs"

        if os.path.exists(imagedir + "/squashed.tar"):
            return (True)

        if not self.anchore_layers:
            return (False)

        if not os.path.exists(rootfsdir):
            os.makedirs(rootfsdir)

        revlayer = list(self.anchore_layers)
        revlayer.reverse()

        excludesfile = '/'.join([imagedir, 'tarexcludes'])
        open(excludesfile, 'w').close()

        for l in revlayer:
            layertar = imagedir + "/" + l + "/layer.tar"
            self._logger.debug("layer to squash: " + layertar)

            tarcmd = ["tar", "-C", rootfsdir, "-t", "-f", layertar]
            self._logger.debug("cmd: " + ' '.join(tarcmd))

            allfiles = subprocess.check_output(tarcmd)
            OFH=open(excludesfile, 'a')
            for f in allfiles.splitlines():
                if re.match('.*\.wh\..*', f):
                    fsub = re.sub(r"\.wh\.", "", f)
                    OFH.write(f + "\n")
                    OFH.write(fsub + "\n")
            OFH.close()

            tarcmd = ["tar", "-C", rootfsdir, "-X", excludesfile, "-x", "-v", "-f", layertar]
            self._logger.debug("cmd: " + ' '.join(tarcmd))

            try:
                allfiles = subprocess.check_output(tarcmd)
            except Exception as err:
                self._logger.warn("Warn: Untar of unpacked image layer failed - proceeding but not all files in image are present.")
                self._logger.warn("Command: " + ' '.join(tarcmd))
                self._logger.warn("Info: " + str(err))

            OFH=open(excludesfile, 'a')
            for f in allfiles.splitlines():
                OFH.write(f + "\n")
            OFH.close()

        self.squashtar = imagedir + "/squashed.tar"
        self.squashed_allfiles = subprocess.check_output(["tar", "-C", rootfsdir, "-c", "-v", "-f", self.squashtar, "."])
        return (True)

    def squash_orig(self, imagedir=None):
        if not imagedir:
            imagedir = self.tmpdir

        rootfsdir = imagedir + "/rootfs"

        if os.path.exists(imagedir + "/squashed.tar"):
            return (True)

        if not self.anchore_layers:
            return (False)

        if not os.path.exists(rootfsdir):
            os.makedirs(rootfsdir)

        for l in self.anchore_layers:
            layertar = imagedir + "/" + l + "/layer.tar"
            allfiles = subprocess.check_output(["tar", "-C", rootfsdir, "-x", "-v", "-f", layertar])
            for f in allfiles.splitlines():
                patt = re.compile('.*\.wh\..*')
                if (patt.match(f)):
                    fsub = re.sub(r"\.wh\.", "", f)
                    absfiles = list()
                    absfiles.append(rootfsdir + "/" + f)
                    absfiles.append(rootfsdir + "/" + fsub)

                    for absfile in absfiles:
                        if (os.path.exists(absfile)):
                            if (os.path.islink(absfile) or os.path.isfile(absfile)):
                                os.remove(absfile)
                            if (os.path.isdir(absfile)):
                                shutil.rmtree(absfile)

        self.squashtar = imagedir + "/squashed.tar"
        self.squashed_allfiles = subprocess.check_output(
            ["tar", "-C", rootfsdir, "-c", "-v", "-f", self.squashtar, "."])
        return (True)

    def unpack(self, docleanup=True, destdir=None):
        if destdir:
            imagedir = destdir + "/" + str(random.randint(0, 9999999)) + ".anchoretmp"
        else:
            imagedir = self.tmpdir

        shortid = self.meta['shortId']
        imagetar = imagedir + "/image.tar"

        self.docleanup = docleanup

        if not os.path.exists(imagedir):
            os.makedirs(imagedir)

        if not os.path.exists(imagetar):
            FH = open(imagetar, 'w')
            FH.write(self.docker_cli.get_image(shortid).data)
            FH.close()
            sout = subprocess.check_output(["tar", "-C", imagedir, "-x", "-f", imagetar], stderr=DEVNULL)

        self.squash(imagedir)
        return (imagedir)

    def generate_image_report(self):
        # this routine reads the results of image analysis and generates a formatted report
        report = {}

        report['meta'] = {}
        report['docker_data'] = {}
        report['anchore_current_tags'] = []
        report['anchore_all_tags'] = []
        report['familytree'] = []
        report['layers'] = []

        if self.meta: report['meta'] = self.meta
        if self.anchore_current_tags: report['anchore_current_tags'] = self.anchore_current_tags
        if self.anchore_all_tags: report['anchore_all_tags'] = self.anchore_all_tags
        if self.docker_data: report['docker_data'] = self.docker_data
        if self.get_familytree(): report['familytree'] = self.get_familytree()
        if self.get_layers(): report['layers'] = self.get_layers()

        return (report)

    def get_dockerfile_contents(self):
        ret = ["", "NA"]

        modestr = "NA"
        dbuf = ""
        if os.path.exists(self.anchore_imagedir + "/Dockerfile"):
            modestr = "Actual"
            dbuf = self.dockerfile_contents
            #dbuf = anchore_utils.read_plainfile_tostr(self.anchore_imagedir + "/Dockerfile")
        else:
            modestr = "Guessed"
            dbuf = self.dockerfile_contents
            #dbuf = self.discover_dockerfile_contents()

        return ([dbuf, modestr])



#!/usr/bin/env python

import sys
import os
import re
import anchore.anchore_utils

try:
    config = anchore.anchore_utils.init_query_cmdline(sys.argv, "params: <base image ID> <base image ID> ...\nhelp: use 'all' to show all base image IDs")
except:
    sys.exit(1)

if not config:
    sys.exit(0)

outlist = list()
outlist.append(["InputImageId", "InputRepo/Tag", "CurrentBaseId", "CurrentBaseRepo/Tag", "Status", "LatestBaseId", "LatestBaseRepo/Tag"])

allimages = {}
for imageId in config['images']:
    try:

        image = anchore.anchore_image.AnchoreImage(imageId, config['anchore_config']['image_data_store'], allimages)

        bimageId = image.get_earliest_anchore_base()
        if bimageId:
            bimage = anchore.anchore_image.AnchoreImage(bimageId, config['anchore_config']['image_data_store'], allimages)
        else:
            bimage = image

        currtags = bimage.get_alltags_current()
        alltags = bimage.get_alltags_ever()
        checktags = list(set(alltags) - set(currtags))
        uptodate = True
        for t in checktags:
            try:
                cimage = anchore.anchore_image.AnchoreImage(t, config['anchore_config']['image_data_store'], allimages)
                cimageId = cimage.meta['imageId']
                if cimage.meta['imageId'] != bimage.meta['imageId']:
                    uptodate = False
            except:
                pass

        if uptodate:
            if 'all' in config['params'] or bimage.meta['shortId'] in config['params'] or bimage.meta['imageId'] in config['params']:
                outlist.append([image.meta['shortId'], image.meta['humanname'], bimage.meta['shortId'], bimage.get_human_name(), "up-to-date", "N/A", "N/A"])
        else:
            if 'all' in config['params'] or bimage.meta['shortId'] in config['params'] or bimage.meta['imageId'] in config['params']:
                outlist.append([image.meta['shortId'], image.meta['humanname'], bimage.meta['shortId'], bimage.get_human_name(), "out-of-date", cimage.meta['shortId'], cimage.get_human_name()])
    except:
        pass

anchore.anchore_utils.write_kvfile_fromlist(config['output'], outlist)

allimages.clear()
sys.exit(0)



import logging
import sys

# The logging/output philosophy is
#
# stdout - Only the output requested by the user from the commands.
# stderr - Only error or fatal messages or other info as configured by the user
#
# output_verbosity behavior:
#     'quiet': Only output of the command is sent to stdout, stderr is only ERROR messages
#     'normal': quiet + status messages from anchore.console logger to stderr as well as ERRORs
#     'verbose': normal + all INFO-level log messages are sent stderr
#     'debug': verbose+ all DEBUG-level log messages are sent to stderr

# Common variables
DEBUG_FORMAT = '%(asctime)-15s %(levelname)s %(filename)s %(funcName)s %(message)s'
VERBOSE_FORMAT = '%(asctime)-15s %(levelname)s %(message)s'
NORMAL_FORMAT = '%(message)s'
ERR_FORMAT = '%(levelname)s %(message)s' # For errors only

DEBUG_LOGFILE_FORMAT = '%(asctime)-15s %(levelname)s %(filename)s %(funcName)s %(message)s'
LOGFILE_FORMAT = '%(asctime)-15s %(levelname)s %(message)s'

console_verbosity_options = {'quiet': -1,
                             'normal': 0,
                             'verbose': 5,
                             'debug': 10
                             }

class NoTracebackFormatter (logging.Formatter):
    """
    Formatter that does not ever include stacktraces for exception output.
    """

    def __init__(self, fmt=None, datefmt=None, err_fmt=None):
        super(NoTracebackFormatter, self).__init__(fmt=fmt, datefmt=datefmt)
        self._err_format = err_fmt if err_fmt is not None else fmt

    def format(self, record):
        """
        Modified from logging/__init__.py in python 2.7 lib

        Format the specified record as text.

        The record's attribute dictionary is used as the operand to a
        string formatting operation which yields the returned string.
        Before formatting the dictionary, a couple of preparatory steps
        are carried out. The message attribute of the record is computed
        using LogRecord.getMessage(). If the formatting string uses the
        time (as determined by a call to usesTime(), formatTime() is
        called to format the event time. If there is exception information,
        it is formatted using formatException() and appended to the message.
        """
        record.message = record.getMessage()
        if self.usesTime():
            record.asctime = self.formatTime(record, self.datefmt)

        if record.levelno >= logging.getLevelName('ERROR'):
            s = self._err_format % record.__dict__
        else:
            s = self._fmt % record.__dict__

        if record.exc_info:
            record.exc_text = self.formatException(record.exc_info)
        if record.exc_text:
            # Trim the trailing newline
            if s[-1:] == "\n":
                s = s[:-1]

            try:
                # Delimit with the colon
                s = s + ': ' + record.exc_text
            except UnicodeError:
                # Sometimes filenames have non-ASCII chars, which can lead
                # to errors when s is Unicode and record.exc_text is str
                # See issue 8924.
                # We also use replace for when there are multiple
                # encodings, e.g. UTF-8 for the filesystem and latin-1
                # for a script. See issue 13232.
                s = s + record.exc_text.decode(sys.getfilesystemencoding(),
                                               'replace')
            # Reset to avoid using the cache since this logger changes the exception format
            record.exc_text = None
        return s

    def formatException(self, ei):
        return '%s' % (ei[1])


class LoggerNamePrefixFilter(logging.Filter):
    """
    Filters out all non-Anchore records by logger name.
    """

    def __init__(self, name='', prefix=None, non_match_loglevel='ERROR'):
        super(LoggerNamePrefixFilter, self).__init__(name)
        self.non_match_level = non_match_loglevel
        self._prefix = prefix

    def filter(self, record):
        return record.name.startswith(self._prefix) or \
               record.levelno >= logging.getLevelName(self.non_match_level)


def init_output_formatters(output_verbosity='normal', stderr=sys.stderr, logfile=None, debug_logfile=None):
    """
    Initialize the CLI logging scheme.

    :param output_verbosity: 'quiet','normal','verbose', or 'debug' controls the output to stdout and its format
    :param stderr: stream for stderr output, default=stderr, pass a file path/string to have stderr in a file
    :return:
    """

    if output_verbosity not in console_verbosity_options:
        raise ValueError('output_verbosity must be one of: %s' % console_verbosity_options.keys())

    # Initialize debug log file, 'anchore-debug.log'. This log has stack-traces and is expected to be human read
    # and intended for developers and debugging, not an operational log.

    # Configure stderr behavior. All errors go to screen
    stderr_handler = logging.StreamHandler(stderr)

    if output_verbosity == 'quiet':
        stderr_handler.setLevel(level='ERROR')
        stderr_handler.setFormatter(NoTracebackFormatter(fmt=NORMAL_FORMAT, err_fmt=ERR_FORMAT))
        logging.root.setLevel('ERROR')  # Allow all at top level, filter specifics for each handler

    elif output_verbosity == 'normal':
        # The specific console logger
        stderr_handler.setLevel('INFO')
        stderr_formatter = NoTracebackFormatter(fmt=NORMAL_FORMAT, err_fmt=ERR_FORMAT)
        stderr_handler.setFormatter(stderr_formatter)
        stderr_handler.addFilter(LoggerNamePrefixFilter(prefix='anchore', non_match_loglevel='ERROR'))
        logging.root.setLevel('INFO')

    elif output_verbosity == 'verbose':
        stderr_handler.setLevel('INFO')
        stderr_handler.setFormatter(NoTracebackFormatter(fmt=NORMAL_FORMAT, err_fmt=ERR_FORMAT))
        logging.root.setLevel('INFO')

    elif output_verbosity == 'debug':
        stderr_handler.setLevel(level='DEBUG')
        stderr_handler.setFormatter(logging.Formatter(fmt=DEBUG_FORMAT))
        logging.root.setLevel('DEBUG')

    logging.root.addHandler(stderr_handler)

    if debug_logfile:
        debug_filehandler = logging.FileHandler(debug_logfile)
        debug_filehandler.setLevel('DEBUG')
        formatter = logging.Formatter(fmt=DEBUG_LOGFILE_FORMAT)
        debug_filehandler.setFormatter(formatter)
        logging.root.addHandler(debug_filehandler)
        logging.root.setLevel('DEBUG')

    if logfile:
        filehandler = logging.FileHandler(logfile)
        filehandler.setLevel('INFO')
        filehandler.setFormatter(NoTracebackFormatter(fmt=LOGFILE_FORMAT, err_fmt=LOGFILE_FORMAT))
        logging.root.addHandler(filehandler)


if __name__ == '__main__':
    code_debug = True
    print 'Running local tests'
    init_output_formatters('normal', stderr=sys.stderr, debug_logfile='./test_debug.log')

    log = logging.getLogger('anchore.test')
    log.debug('Some debug msg')
    log.info('Some info msg')
    log.error('Some error msg')
    try:
        raise StandardError('Some error exception')
    except:
        log.exception('caught exception')

    print 'Tests done'